"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.sourceNodes = void 0;
const tslib_1 = require("tslib");
const package_json_1 = tslib_1.__importDefault(require("../../package.json"));
const database_1 = require("../lib/database");
const nodes_1 = require("../lib/nodes");
function sourceNodes(sourceNodeArgs, options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { reporter } = sourceNodeArgs;
        try {
            reporter.info(`Sourcing data using "${package_json_1.default.name}" @ ${package_json_1.default.version}`);
            const { connection, queryBatchSize, queries } = options;
            const dataApi = database_1.getDataApi(connection);
            const queryResults = yield database_1.performQueries(sourceNodeArgs, dataApi, queries, queryBatchSize);
            const nodes = nodes_1.prepareNodes(sourceNodeArgs, connection.databaseName, queryResults);
            nodes_1.createNodes(sourceNodeArgs, nodes);
            reporter.success(`Successfully sourced ${nodes.length} node(s) using "${package_json_1.default.name}"`);
        }
        catch (e) {
            const err = e;
            reporter.error(`Error while sourcing data with "${package_json_1.default.name}"`, err);
        }
    });
}
exports.sourceNodes = sourceNodes;
