"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.performQueries = exports.getDataApi = void 0;
const tslib_1 = require("tslib");
const aurora_data_api_1 = tslib_1.__importDefault(require("aurora-data-api"));
const utilities_1 = require("./utilities");
const DEFAULT_MAX_QUERY_BATCH_SIZE = 10;
function getDataApi(connection) {
    return new aurora_data_api_1.default({
        accessKeyId: connection.accessKeyId,
        secretAccessKey: connection.secretAccessKey,
        region: connection.region,
        resourceArn: connection.resourceArn,
        secretArn: connection.secretArn,
        database: connection.databaseName,
    });
}
exports.getDataApi = getDataApi;
function performQuery(dataApi, query) {
    var _a;
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const opts = {};
        if (query.databaseName)
            opts.database = query.databaseName;
        const results = yield dataApi.query(query.statement, {}, opts);
        return {
            query,
            rows: (_a = results.rows) !== null && _a !== void 0 ? _a : [],
        };
    });
}
function performQueries(sourceNodeArgs, dataApi, queries, queryBatchSize = DEFAULT_MAX_QUERY_BATCH_SIZE) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { reporter } = sourceNodeArgs;
        reporter.info(`Executing ${queries.length} queries`);
        const batchedQueries = utilities_1.chunkArray(queries, queryBatchSize);
        const output = [];
        for (const batch of batchedQueries) {
            const promises = batch.map(query => performQuery(dataApi, query));
            const results = yield Promise.all(promises);
            output.push(...results);
        }
        return output;
    });
}
exports.performQueries = performQueries;
