"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createNodes = exports.prepareNodes = void 0;
const NODE_TYPE = `serverless-aurora`;
function flattenQueryResultsToNodeList(sourceNodeArgs, queryResults) {
    const { createContentDigest, createNodeId } = sourceNodeArgs;
    const nodeList = [];
    for (const { query, rows } of queryResults) {
        const idFieldName = query.idFieldName || `id`;
        const nodeType = `${NODE_TYPE}-${query.nodeName}`;
        for (const row of rows) {
            nodeList.push({
                data: Object.assign(Object.assign({}, row), { id: createNodeId(`${nodeType}-${row[idFieldName]}`), links: { parents: {}, children: {} }, internal: {
                        type: nodeType,
                        content: JSON.stringify(row),
                        contentDigest: createContentDigest(row),
                    } }),
                query,
                row,
            });
        }
    }
    return nodeList;
}
function findParentNodes(parentNodeName, parentMatcher, childNode, nodeList) {
    let parentNodes = parentNodeName
        ? nodeList.filter(node => node.query.nodeName === parentNodeName)
        : nodeList;
    if (parentMatcher)
        parentNodes = parentNodes.filter(parentNode => parentMatcher(childNode.row, parentNode.row));
    return parentNodes;
}
function addForignKeyRelationships(nodeList) {
    for (const childNode of nodeList) {
        const { nodeName: childNodeName, parentNodeName, parentMatcher } = childNode.query;
        const parentNodes = findParentNodes(parentNodeName, parentMatcher, childNode, nodeList);
        if (!parentNodes.length)
            continue;
        const childToParentLinks = [];
        const parentToChildKey = `${childNodeName}___NODE`;
        parentNodes.forEach(parentNode => {
            var _a;
            childToParentLinks.push(parentNode.data.id);
            parentNode.data.links.children[parentToChildKey] = (_a = parentNode.data.links.children[parentToChildKey]) !== null && _a !== void 0 ? _a : [];
            parentNode.data.links.children[parentToChildKey].push(childNode.data.id);
        });
        childNode.data.links.parents[`${parentNodeName}___NODE`] = childToParentLinks;
    }
}
function prepareNodes(sourceNodeArgs, queryResults) {
    const nodeList = flattenQueryResultsToNodeList(sourceNodeArgs, queryResults);
    addForignKeyRelationships(nodeList);
    return nodeList;
}
exports.prepareNodes = prepareNodes;
function createNodes(sourceNodeArgs, nodeList) {
    const { actions } = sourceNodeArgs;
    nodeList.forEach(node => actions.createNode(node.data));
}
exports.createNodes = createNodes;
