"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const package_json_1 = tslib_1.__importDefault(require("../package.json"));
const database_1 = require("./database");
const nodes_1 = require("./nodes");
function sourceNodes(sourceNodeArgs, options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { reporter } = sourceNodeArgs;
        try {
            reporter.info(`Sourcing data from Serverless Aurora database`);
            const { connectionDetails, queryBatchSize, queries } = options;
            const dataApi = database_1.getDataApi(connectionDetails);
            const queryResults = yield database_1.performQueries(dataApi, queries, queryBatchSize);
            const nodes = nodes_1.prepareNodes(sourceNodeArgs, queryResults);
            nodes_1.createNodes(sourceNodeArgs, nodes);
            reporter.success(`Successfully sourced ${nodes.length} node(s) from Serverless Aurora database`);
        }
        catch (e) {
            const err = e;
            reporter.error(`Error while sourcing data with "${package_json_1.default.name}"`, err);
        }
    });
}
exports.default = sourceNodes;
